package tech.sbdevelopment.themeparkaudio.socket.messages;

import lombok.Getter;
import tech.sbdevelopment.themeparkaudio.ThemeParkAudio;
import tech.sbdevelopment.themeparkaudio.api.AudioType;
import org.json.simple.JSONObject;

import javax.net.ssl.HttpsURLConnection;
import java.io.IOException;
import java.net.HttpURLConnection;
import java.net.URI;
import java.net.URISyntaxException;

@Getter
public class AudioMessage extends AbstractMessage {
    private final String path;
    private final Boolean play;
    private final Integer at;

    protected AudioMessage(AudioType type, String path, Boolean play, Integer at) {
        super(type.toMessageTask());
        this.path = parsePath(path);
        this.play = play;
        this.at = at;
    }

    public static AudioMessage of(AudioType type, String path) {
        return new AudioMessage(type, path, null, null);
    }

    public static AudioMessage of(AudioType type, String path, boolean play) {
        return new AudioMessage(type, path, play, null);
    }

    public static AudioMessage of(AudioType type, String path, boolean play, int at) {
        return new AudioMessage(type, path, play, at);
    }

    private static String parsePath(String path) {
        if (path.startsWith("https://mcau.nl")) {
            try {
                HttpsURLConnection connection = (HttpsURLConnection) new URI(path).toURL().openConnection();
                connection.setInstanceFollowRedirects(false);
                connection.connect();
                if (connection.getResponseCode() == HttpURLConnection.HTTP_MOVED_PERM ||
                        connection.getResponseCode() == HttpURLConnection.HTTP_MOVED_TEMP) {
                    path = connection.getHeaderField("Location");
                }
            } catch (URISyntaxException | IOException e) {
                ThemeParkAudio.getInstance().getLogger().warning("Failed to resolve URL: " + e.getMessage());
            }
        }
        return path;
    }

    @Override
    protected JSONObject extendJson() {
        JSONObject data = new JSONObject();
        data.put("path", path);
        if (play != null) data.put("play", play);
        if (at != null) data.put("at", at);
        return data;
    }
}
